@@ -12,6 +12,7 @@ extern crate diesel_migrations;
 #[cfg(test)]
 extern crate serial_test;
 
+use chrono::NaiveDateTime;
 use diesel::{result::Error, *};
 use lemmy_db_schema::{CommunityId, DbUrl, PersonId};
 use lemmy_utils::ApiError;
@@ -130,13 +131,22 @@ pub trait Readable {
 
 pub trait Reportable {
   type Form;
+  type IdType;
   fn report(conn: &PgConnection, form: &Self::Form) -> Result<Self, Error>
   where
     Self: Sized;
-  fn resolve(conn: &PgConnection, report_id: i32, resolver_id: PersonId) -> Result<usize, Error>
+  fn resolve(
+    conn: &PgConnection,
+    report_id: Self::IdType,
+    resolver_id: PersonId,
+  ) -> Result<usize, Error>
   where
     Self: Sized;
-  fn unresolve(conn: &PgConnection, report_id: i32, resolver_id: PersonId) -> Result<usize, Error>
+  fn unresolve(
+    conn: &PgConnection,
+    report_id: Self::IdType,
+    resolver_id: PersonId,
+  ) -> Result<usize, Error>
   where
     Self: Sized;
 }
@@ -145,14 +155,14 @@ pub trait DeleteableOrRemoveable {
   fn blank_out_deleted_or_removed_info(self) -> Self;
 }
 
+// TODO: move this to apub lib
 pub trait ApubObject {
-  type Form;
+  /// If this object should be refetched after a certain interval, it should return the last refresh
+  /// time here. This is mainly used to update remote actors.
+  fn last_refreshed_at(&self) -> Option<NaiveDateTime>;
   fn read_from_apub_id(conn: &PgConnection, object_id: &DbUrl) -> Result<Self, Error>
   where
     Self: Sized;
-  fn upsert(conn: &PgConnection, user_form: &Self::Form) -> Result<Self, Error>
-  where
-    Self: Sized;
 }
 
 pub trait MaybeOptional<T> {
@@ -266,7 +276,7 @@ pub fn diesel_option_overwrite_to_url(
     Some("") => Ok(Some(None)),
     Some(str_url) => match Url::parse(str_url) {
       Ok(url) => Ok(Some(Some(url.into()))),
-      Err(_) => Err(ApiError::err("invalid_url")),
+      Err(e) => Err(ApiError::err("invalid_url", e)),
     },
     None => Ok(None),
   }
